var recur = require('../lib/recur').recur;
var cron = require('../lib/cron.parser').cronParser;
var text = require('../lib/en.parser').enParser;
var later = require('../lib/later').later;
var should = require('should');

describe('Examples', function() {

	describe('schedules', function() {

		var rSched, cSched, tSched, mSched;

		it('every 5 minutes', function() {

			rSched = recur().every(5).minute();
			cSched = cron().parse('* */5 * * * *', true);
			tSched = text().parse('every 5 minutes');
			mSched = {schedules: [ {m: [0,5,10,15,20,25,30,35,40,45,50,55]}]};

			rSched.schedules[0].should.eql(cSched.schedules[0]);
			cSched.schedules[0].should.eql(tSched.schedules[0]);
			tSched.schedules[0].should.eql(mSched.schedules[0]);
		});

		it('every 5 minutes on the 0th second', function() {

			rSched = recur().every(5).minute().first().second();
			cSched = cron().parse('0 */5 * * * *', true);
			tSched = text().parse('every 5 minutes on the first second');
			mSched = {schedules: [ {s: [0], m: [0,5,10,15,20,25,30,35,40,45,50,55]}]};

			rSched.schedules[0].should.eql(cSched.schedules[0]);
			cSched.schedules[0].should.eql(tSched.schedules[0]);
			tSched.schedules[0].should.eql(mSched.schedules[0]);
		});

		it('every Tuesday at at 11 am', function() {

			rSched = recur().on(3).dayOfWeek().at('11:00');
			cSched = cron().parse('* 0 11 * * 2', true);  // not quite equivalent
			tSched = text().parse('on tuesday at 11:00 am');
			mSched = {schedules: [ {d: [3], t: ['11:00:00']}]};
			var mSched2 = {schedules: [ { m: [ 0 ], h: [ 11 ], d: [ 3 ] }]};

			rSched.schedules[0].should.eql(tSched.schedules[0]);
			tSched.schedules[0].should.eql(mSched.schedules[0]);
			cSched.schedules[0].should.eql(mSched2.schedules[0]);
		});

		it('every hour on weekdays and every other hour on weekends', function() {

			rSched = recur().every(1).hour().onWeekday().and().every(2).hour().onWeekend();
			tSched = text().parse('every 1 hour every weekday also every 2 hours every weekend');
			mSched = {schedules: [ 
				{h: [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23], 
				 d:[2,3,4,5,6]},
				{h: [0,2,4,6,8,10,12,14,16,18,20,22], 
				 d:[1,7]}]};

			rSched.schedules[0].should.eql(tSched.schedules[0]);
			tSched.schedules[0].should.eql(mSched.schedules[0]);
			rSched.schedules[1].should.eql(tSched.schedules[1]);
			tSched.schedules[1].should.eql(mSched.schedules[1]);
		});

	});
});